@@ -11,7 +11,7 @@ Expected keys:
 """
 import glob
 try:
-    import configparser as configparser 
+    import ConfigParser as configparser 
 except ImportError:
     import configparser
 
