@@ -30,6 +30,7 @@ def clean( target=DEFAULT_FIRMWARE_DIRECTORY, protected = None ):
             for p in PROTECTED
         ]
     # current target is always protected...
+    target = final_path(target)
     current = os.path.join( target, CURRENT_LINK)
     assert os.path.exists( current ), "Current link appears to be missing, corrupt installation  (e.g. run install)?"
     current_target = final_path( current )
@@ -73,7 +74,7 @@ def swap_link( final_target, current ):
         # cron-locked, but that's not obvious here...
     except (OSError,IOError):
         pass
-    os.symlink( final_target, tmp )
+    os.symlink( os.path.basename(final_target), tmp )
     os.rename( tmp, current )
 
 def install_bytes( filename, keyring='/etc/fussy/keys', target='/opt/firmware' ):
@@ -153,7 +154,7 @@ def enable( final_target, current, recovery=False ):
             pipe = nbio.Process( [
                 pre_install,
                 final_target,
-                bytes(recovery),
+                unicode(recovery).encode('utf-8'),
             ], by_line=True, stderr=-1) | report_progress
             pipe()
         log.info( 'Setting firmware current' )
@@ -165,7 +166,7 @@ def enable( final_target, current, recovery=False ):
             shutil.rmtree( final_target, True )
             raise
         else:
-            log.error( 'Failure attempting to re-initialize previous installation, likely reboot required' )
+            log.exception( 'Failure attempting to re-initialize previous installation, likely reboot required' )
             # Note, we will attempt to run post-install, as it is often able to perform 
             # cleanups that the pre-install cannot...
     
@@ -176,7 +177,7 @@ def enable( final_target, current, recovery=False ):
         pipe = nbio.Process([
             post_install,
             final_target,
-            bytes(recovery),
+            unicode(recovery).encode('utf-8'),
         ], by_line=True, stderr=-1) | report_progress
         pipe()
 
@@ -190,7 +191,7 @@ def ensure_current_link( current, failsafe ):
             # this is the initial install case, where we want failsafe to be 
             # created from the initial installation image...
             os.rename( current, failsafe )
-            os.symlink( failsafe, current )
+            os.symlink( os.path.basename(failsafe), current )
         else:
             raise RuntimeError(
                 """%(current)r is a directory, it should be a link.  Failsafe already exists."""%locals()
