@@ -5,7 +5,7 @@ pipe = open( somefile, 'rb' ) | nbio.Process( ['grep','blue'] ) | nbio.Process(
 """
 import os, select, fcntl, time, sys, logging, subprocess, errno, collections
 log = logging.getLogger( __name__ )
-from ._shims import bytes, unicode
+from ._shims import bytes, unicode, file_base, next, long
 
 DEFAULT_READ_BLOCK = 1024*64
 
@@ -91,19 +91,18 @@ class Pipe( object ):
                 something_happened = 0
                 for iterable in iterables:
                     try:
-                        child_result = iterable.next()
+                        child_result = next(iterable)
                         if child_result is DID_NOTHING:
                             # we did nothing during this iteration
                             pass
                         elif child_result:
                             something_happened += 1
                             yield child_result
-                    except StopIteration as err:
+                    except StopIteration:
                         exhausted.append( iterable )
-                    except Exception as err:
+                    except (KeyboardInterrupt,Exception):
                         # Have the *first* item that failed raise the error, 
                         # and only if they don't, raise this error...
-                        log.error( 'Catching error: %s', err )
                         for item in self.processes:
                             item.check_exit()
                         raise
@@ -116,7 +115,14 @@ class Pipe( object ):
                     while done in iterables:
                         iterables.remove( done )
                 if not something_happened:
-                    pause( self.pause_on_silence )
+                    try:
+                        pause( self.pause_on_silence )
+                    except (KeyboardInterrupt,Exception) as err:
+                        new_err = ProcessError( 
+                            "Exception while paused: %s"%(err,)
+                        )
+                        new_err.process = self 
+                        raise new_err
                 if timeout:
                     if time.time() > until:
                         raise Timeout( self, timeout )
@@ -131,8 +137,8 @@ class Pipe( object ):
             for item in self.__iter__( timeout=timeout ):
                 if item:
                     result.append( item )
-            return "".join( result )
-        except Exception as err:
+            return b"".join( result )
+        except (KeyboardInterrupt,Exception) as err:
             err.output = result
             raise
     def append( self, process ):
@@ -215,7 +221,7 @@ class Pipe( object ):
         """
         if isinstance( other, PipeComponent ):
             pass 
-        elif isinstance( other, file ):
+        elif isinstance( other, file_base ):
             other = FileComponent( other )
         elif isinstance( other, (bytes,unicode)):
             if other == '-':
@@ -236,7 +242,7 @@ class Pipe( object ):
             if hasattr( process, 'kill' ):
                 try:
                     process.kill()
-                except Exception:
+                except (KeyboardInterrupt,Exception):
                     log.warn(
                         "Unable to kill process: %s", process,
                     )
@@ -253,7 +259,7 @@ class PipeComponent( object ):
             exhausted = []
             for iterable in iterables:
                 try:
-                    child_result = iterable.next()
+                    child_result = next(iterable)
                     if child_result is DID_NOTHING:
                         pass
                     elif child_result:
@@ -261,7 +267,7 @@ class PipeComponent( object ):
                         yield child_result
                 except StopIteration as err:
                     exhausted.append( iterable )
-                except Exception as err:
+                except (KeyboardInterrupt,Exception) as err:
                     err.args += (self,)
                     raise 
             for done in exhausted:
@@ -285,7 +291,7 @@ class FileComponent( PipeComponent ):
         self.stdout = fileout or filein 
         super( FileComponent, self ).__init__()
     def iter_read( self ):
-        return reader( self.stdout )
+        return reader( self.stdout, exit_on_finish=True )
     def iter_write( self, source ):
         return writeiter(
             source,
@@ -653,6 +659,8 @@ def writer( content,fh, encoding='utf8', block_size=4096 ):
     
     Used by writeiter to writing individual bits of content to the fh
     """
+    if hasattr( fh, 'raw' ):
+        fh = fh.raw
     fp = fileno( fh )
     if isinstance( content, unicode ):
         content = content.encode( encoding )
@@ -679,9 +687,11 @@ def writer( content,fh, encoding='utf8', block_size=4096 ):
                 block = block[written:]
                 yield written
         
-def reader( fh, blocksize = DEFAULT_READ_BLOCK ):
+def reader( fh, blocksize = DEFAULT_READ_BLOCK, exit_on_finish=False ):
     """Produce content blocks from fh without blocking
     """
+    if hasattr( fh, 'raw' ):
+        fh = fh.raw
     try:
         fd = fileno( fh )
     except ValueError as err:
@@ -691,7 +701,10 @@ def reader( fh, blocksize = DEFAULT_READ_BLOCK ):
             rr,rw,re = select.select([fh],[],[],0)
             if rr:
                 result = os.read(fd, blocksize )
-                if result == '':
+                if result == b'':
+                    if exit_on_finish:
+                        close(fh)
+                        return
                     break
             else:
                 result = DID_NOTHING
@@ -710,6 +723,8 @@ def reader( fh, blocksize = DEFAULT_READ_BLOCK ):
 
 def fileno( fh ):
     "Determine the fileno for the file-like thing"
+    if hasattr( fh, 'raw' ):
+        fh = fh.raw
     if hasattr( fh, 'fileno' ):
         return fh.fileno()
     return fh
@@ -729,18 +744,18 @@ def close( fh ):
         raise RuntimeError( "Child process returned error code", errcode )
 def by_line( iterable ):
     """Buffer iterable yielding individual lines"""
-    buffer = ""
+    buffer = b""
     for item in iterable:
         if not item:
             yield item 
         else:
             buffer += item 
-            if '\n' not in item:
+            if b'\n' not in item:
                 # We did something, but we're not ready to yield a real value...
                 yield None
             else:
-                while '\n' in buffer:
-                    line, buffer = buffer.split('\n',1)
+                while b'\n' in buffer:
+                    line, buffer = buffer.split(b'\n',1)
                     yield line 
     if buffer:
         yield buffer 
