@@ -1,5 +1,6 @@
 """Transactionally write a file to disk"""
 import os,logging
+from ._shims import bytes, unicode
 log = logging.getLogger( __name__ )
 
 def twrite( filename, content ):
@@ -12,12 +13,12 @@ def twrite( filename, content ):
         Transactionality is Linux specific (os.rename guarantee)
     """
     directory = os.path.dirname( filename )
-    if not os.path.exists( directory ):
+    if directory and not os.path.exists( directory ):
         log.debug( 'Creating directory: %s', directory )
         os.makedirs( directory )
     tmp = os.path.join( filename+'~' )
     fh = open( tmp, 'wb' )
-    if isinstance( content, str ):
+    if isinstance( content, bytes ):
         fh.write( content )
     elif isinstance( content, unicode ):
         fh.write( content.encode( 'utf-8' ))
@@ -26,4 +27,11 @@ def twrite( filename, content ):
             fh.write( item )
     fh.close()
     log.debug( 'Writing file: %s', filename )
-    os.rename( tmp, filename )
+    try:
+        os.rename( tmp, filename )
+    except OSError as err:
+        if err.errno == 2: # no such directory
+            pass
+        else:
+            raise 
+    
